# -*- coding: utf-8 -*-

# Almost all code comes from http://code.activestate.com/recipes/552751/

import sys
import json
import select 
import threading
import traceback
import xmlrpc.client
try:
    import fcntl
except ImportError:
    fcntl = None


class ResponseError(xmlrpc.client.ResponseError):
    pass


class Fault(xmlrpc.client.ResponseError):
    pass


def _get_response(file, sock):
    data = b""
    while 1:
        if sock:
            response = sock.recv(1024)
        else:
            response = file.read(1024)
        if not response:
            break
        data += response

    file.close()

    return data.decode("utf_8")


if sys.version_info[1] == 0:
    class Transport(xmlrpc.client.Transport):
        def _parse_response(self, file, sock):
            return _get_response(file, None)


    class SafeTransport(xmlrpc.client.SafeTransport):
        def _parse_response(self, file, sock):
            return _get_response(file, None)
else:
    class Transport(xmlrpc.client.Transport):
        def parse_response(self, file):
            return _get_response(file, None)


    class SafeTransport(xmlrpc.client.SafeTransport):
        def parse_response(self, file):
            return _get_response(file, None)


class ServerProxy:
    def __init__(self, uri, id=None, transport=None, use_datetime=0):
        # establish a "logical" server connection

        # get the url
        import urllib.parse
        type, uri = urllib.parse.splittype(uri)
        if type not in ("http", "https"):
            raise IOError("unsupported JSON-RPC protocol")
        self.__host, self.__handler = urllib.parse.splithost(uri)
        if not self.__handler:
            self.__handler = "/JSON"

        if transport is None:
            if type == "https":
                transport = SafeTransport(use_datetime=use_datetime)
            else:
                transport = Transport(use_datetime=use_datetime)

        self.__transport = transport
        self.__id        = id

    def __request(self, methodname, params):
        # call a method on the remote server

        request_raw = json.dumps(dict(id=self.__id, method=methodname,
                                      params=params), 
                                      ensure_ascii=False).encode("utf-8")
        response_raw = self.__transport.request(
            self.__host,
            self.__handler,
            request_raw,
            verbose=False)
        response = json.loads(response_raw)

        if response["id"] != self.__id:
            raise ResponseError("Invalid request id (is: %s, expected: %s)" \
                                % (response["id"], self.__id))
        if response["error"] is not None:
            raise Fault("JSON Error", response["error"])
        return response["result"]

    def __repr__(self):
        return "<ServerProxy for %s%s>" % (self.__host, self.__handler)

    __str__ = __repr__

    def __getattr__(self, name):
        return xmlrpc.client._Method(self.__request, name)



# -*- coding: utf-8 -*-

import unittest
berrymq = None


class TestPriorityQueue(unittest.TestCase):
    def test_priority_queue(self):
        queue = berrymq._PriorityQueue()
        queue.put("b", 10)
        queue.put("a", 5)
        self.assertEqual((5, "a"), queue.get())
        self.assertEqual((10, "b"), queue.get())


class TestQueue(unittest.TestCase):
    def setUp(self):
        try:
            import queue
        except ImportError:
            import Queue as queue
        self.queue = berrymq.Queue(None)
        self.queue._backend_queue = queue.Queue()

    def test_empty_queue(self):
        self.assertEqual(0, self.queue.qsize())
        self.assert_(self.queue.empty())
        self.assert_(not self.queue.full())
        self.assertRaises(berrymq.Empty, self.queue.get, False)
        self.assertRaises(berrymq.Empty, self.queue.get_nowait)

    def test_not_empty_queue(self):
        self.queue._backend_queue.put((10, "sampledata"))
        self.assertEqual(1, self.queue.qsize())
        self.assert_(not self.queue.empty())
        self.assert_(not self.queue.full())
        self.assertEqual("sampledata", self.queue.get())


class TestPullAPI(unittest.TestCase):
    def test_standalone_queue_object_api(self):
        queue = berrymq.Queue("pull_api2:*")
        berrymq.twitter("pull_api2:test", "arg1", kwargs="kwargs")
        self.assertEqual(1, queue.qsize())
        message = queue.get()
        self.assertEqual("pull_api2", message.name)
        self.assertEqual(set(["test"]), message.action)
        self.assertEqual("arg1", message.args[0])
        self.assertEqual("kwargs", message.kwargs["kwargs"])


def test_setup(berrymq_module):
    global berrymq
    berrymq = berrymq_module
    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.makeSuite(TestQueue))
    test_suite.addTest(unittest.makeSuite(TestPullAPI))
    return test_suite



# -*- coding: utf-8 -*-

import sys
import unittest


def show_usage(alert_version=False):
    print("""berryMQ tests
  for Python 3.0, 3.1 (current interpreter: %d.%d.%d)

  run_tests.py [option] target:

  *option
    --single-thread:  simple version(good for code reading)

  *target
    --all:      run all test
    --basic:    test basic features only
    --pull:     test pull API
    --parallel: test multithreading feature only(slow)
    --jsonrpc:  test jsonrpc components(slow)
    --1st_node: network test(run this first)
    --2nd_node: network test(run this later)
""" % sys.version_info[:3])
    if alert_version:
        print("This test is able to be run on Python 3.X")
    sys.exit()

def main():
    if sys.version_info[0] != 3:
        show_usage(alert_version=True)

    if len(sys.argv) == 1 or "--help" in sys.argv or "-h" in sys.argv:
        show_usage()

    if "--1st_node" in sys.argv:
        import berrymq
        import tests.test_berrymq_network_1st_node
        tests.test_berrymq_network_1st_node.test(mqas)
        sys.exit()
    if "--2nd_node" in sys.argv:
        import berrymq
        import tests.test_berrymq_network_2nd_node
        tests.test_berrymq_network_2nd_node.test(berrymq)
        sys.exit()

    suite = unittest.TestSuite()

    if "--single-thread" in sys.argv:
        import berrymq_singlethread.berrymq as berrymq
    else:
        import berrymq.berrymq as berrymq

    if "--basic" in sys.argv or "--all" in sys.argv:
        import tests.test_berrymq
        suite.addTest(tests.test_berrymq.test_setup(berrymq))

    if "--pull" in sys.argv or "--all" in sys.argv:
        import tests.test_berrymq_pull
        suite.addTest(tests.test_berrymq_pull.test_setup(berrymq))

    if "--parallel" in sys.argv or "--all" in sys.argv:
        import tests.test_berrymq_parallel
        suite.addTest(tests.test_berrymq_parallel.test_setup(berrymq))

    if "--jsonrpc" in sys.argv or "--all" in sys.argv:
        import tests.test_jsonrpc
        suite.addTest(tests.test_jsonrpc.test_setup())

    if suite.countTestCases():
        runner = unittest.TextTestRunner()
        runner.run(suite)
    else:
        show_usage()


if __name__ == "__main__":
    main()





# -*- coding: utf-8 -*-

import re
import sys
import heapq
import queue
import types
import weakref
import threading
import xmlrpc.client
import itertools
import functools


def _dummy(message):
    """Dummy guard condition function.

    It always matches.

    @param message: passed values from expositions.
    @type  message: Message object.
    @return: whether follower should be called or not.
    @rtype : boolean.
    """
    return True


class Identifier:
    """Event naming identifier.

    identifier is built by string.
    It supports wildcard.

    "sample:*" => match any action in "sample"
    "*:entry" => match any "entry" messsge

    @future: including URI(for external followable node)
    """
    __slots__ = ("name", "action", "namespace", "functions")
    _split_name_action = re.compile(r"(.*):(.*)")
    _split_namespace = re.compile(r"(.*)@(.*)")

    def __init__(self, key_or_identifier, action=None, guard_condition=_dummy):
        self.functions = [None, None]
        if action is not None:
            self.name = key_or_identifier.name
            self.namespace = key_or_identifier.namespace
            self.action = set([action])
            self.functions[0] = self._match_all
            return
        name, namespace, action = self._split_key(key_or_identifier)
        self.namespace = namespace
        if name == "*" and action == "*":
            self.name = None
            self.action = None
            self.functions[0] = self._match_always
        elif name == "*" and action != "*":
            self.name = None
            self.action = set([action])
            self.functions[0] = self._match_action_only
        elif name != "*" and action == "*":
            self.name = name
            self.action = None
            self.functions[0] = self._match_name_only
        elif name != "*" and action != "*":
            self.name = name
            self.action = set([action])
            self.functions[0] = self._match_all
        self.functions[1] = guard_condition

    def _split_key(self, key):
        match = self._split_name_action.match(key)
        if match:
            name, action = match.groups()
        else:
            name = key
            action = "*"
        match = self._split_namespace.match(name)
        if match:
            name, namespace = match.groups()
        else:
            namespace = None
        return name, namespace, action

    def __str__(self):
        return "<berrymq.Identifier object at %d: id=%s>" % (
            id(self), self.id_str())

    def id_str(self):
        name = "*" if self.name is None else self.name
        action = "*" if self.action is None else ",".join(sorted(self.action))
        return "%s:%s" % (name, action)

    def _match_all(self, rhs):
        return self._match_name_only(rhs) and self._match_action_only(rhs)

    def _match_name_only(self, rhs):
        if rhs.name is None:
            return True
        return self.name == rhs.name

    def _match_action_only(self, rhs):
        if rhs.action is None:
            return True
        return bool(self.action & rhs.action)

    def _match_always(self, rhs):
        return True

    def is_match(self, expose_identifier, message=None):
        if self.functions[0](expose_identifier):
            return self.functions[1](message)
        return False

    def is_local(self):
        return self.name.startswith("_")


class cond:
    def __init__(self, identifier, guard_condition=_dummy):
        self.targets = [Identifier(identifier, guard_condition=guard_condition)]

    def __and__(self, rhs):
        newtarget = target(None, None)
        newtarget.targets = self.targets + rhs.targets
        return newtarget


def _get_attributes(instance, typeobj):
    """Helper generator to implement TDFW.

    It returns match objects.
    """
    for key in dir(instance):
        if key.startswith("__"):
            continue
        value = getattr(instance, key)
        if type(value) == typeobj:
            yield value


class _weakmethod_ref:
    """This method is arange version of Python Cookbook 2nd 6.10"""
    __slots__ = ("_obj", "_func")
    def __init__(self, fn):
        try:
            o, f = fn.__self__, fn.__func__
        except AttributeError:
            self._obj = None
            self._func = fn
        else:
            self._obj = weakref.ref(o)
            self._func = f

    def __call__(self):
        if self._obj is None:
            return self._func
        elif self._obj() is None:
            return None
        return types.MethodType(self._func, self._obj())


class _weakfunction_ref:
    """This method is arange version of Python Cookbook 2nd 6.10"""
    __slots__ = ("_func")
    def __init__(self, fn):
        self._func = weakref.ref(fn)

    def __call__(self):
        if self._func() is None:
            return None
        return self._func()


class Transporter:
    _followers = {}

    def get_valid_followers(self):
        for name, followers in self._followers.items():
            for id_obj, function_wrapper in followers:
                function = function_wrapper()
                if function is not None:
                    yield id_obj, function

    def regist_follower(self, id_obj, function):
        followers = self._followers.setdefault(id_obj.name, [])
        followers.append((id_obj, function))

    def twitter(self, id_obj, args, kwargs, counter=100):
        self.twitter_local(id_obj, args, kwargs, counter)

    def twitter_local(self, id_obj, args, kwargs, counter=100):
        message = Message(id_obj, args, kwargs, counter)
        if id_obj.name is None:
            actions = self._followers.values()
        else:
            wildcard_actions = self._followers.get(None, [])
            certain_actions = self._followers.get(id_obj.name, [])
            actions = [wildcard_actions, certain_actions]
        for following_id, function in itertools.chain(*actions):
            if not following_id.is_match(id_obj, message):
                continue
            if function() is None:
                continue # delete after
            if ThreadPool.empty():
                function()(message)
            else:
                ThreadPool.request_work(function(), message)


class RootTransporter:
    _default_namespace = None
    _namespaces = {}
    _connections = None

    @classmethod
    def twitter(cls, id_obj, args, kwargs):
        cls.twitter_local(id_obj, args, kwargs)
        id_str = id_obj.id_str()
        if cls._connections:
            cls._connections.forward_message(id_str, args, kwargs)

    @classmethod
    def twitter_local(cls, id_obj, args, kwargs):
        namespace = id_obj.namespace
        if namespace is None:
            namespace = cls._default_namespace
        cls.get(namespace).twitter(id_obj, args, kwargs)

    @classmethod
    def regist_follower(cls, id_obj, method):
        namespace = id_obj.namespace
        if namespace is None:
            namespace = cls._default_namespace
        cls.get(namespace).regist_follower(id_obj, method)

    @classmethod
    def get(cls, namespace):
        message_queue = cls._namespaces.get(namespace)
        if message_queue is None:
            message_queue = Transporter()
            cls._namespaces[namespace] = message_queue
        return message_queue


class ThreadPool:
    _qin = queue.Queue()
    _qerr = queue.Queue()
    _pool = []

    @classmethod
    def _report_error(cls):
        cls._qerr.put(sys.exc_info()[:2])

    @staticmethod
    def _get_all_from_queue(Q):
        try:
            while True:
                yield Q.get_nowait()
        except queue.Empty:
            raise StopIterator

    @classmethod
    def do_work_from_queue(cls):
        while True:
            command, target_method, message = cls._qin.get()
            if command == "stop":
                break
            try:
                if command == "process":
                    target_method(message)
                else:
                    raise ValueError("Unknown command %r" % command)
            except:
                cls._report_error()

    @classmethod
    def make_thread_pool(cls, number):
        if number < 0:
            raise ValueError("'number' should be bigger than 0.")
        number -= len(cls._pool)
        if number > 0:
            for i in range(number):
                new_thread = threading.Thread(target=cls.do_work_from_queue)
                new_thread.setDaemon(True)
                cls._pool.append(new_thread)
                new_thread.start()
        elif number < 0:
            number = abs(number)
            for i in range(number):
                cls.request_work(None, None, "stop")

    @classmethod
    def empty(cls):
        return len(cls._pool) == 0

    @classmethod
    def clear_thread_pool(cls):
        alive_list = []
        for thread in cls._pool:
            if thread.isAlive():
                alive_list.append(thread)
        cls._pool = alive_list


    @classmethod
    def request_work(cls, target_function, message, command="process"):
        cls._qin.put((command, target_function, message))

    @classmethod
    def get_all_errors(cls):
        return cls._get_all_from_queue(cls._qerr)

    @classmethod
    def stop_thread_pool(cls):
        for i in range(len(cls._pool)):
            cls.request_work(None, None, "stop")
        for existing_thread in cls._pool:
            existing_thread.join()
        del cls._pool[:]


class MethodDecorator:
    """This class is attached to exposition/following functions/methods.

    After initialization, this shows any information to draw connections.

    @sa exposition
    @sa following
    @sa following_method
    """
    __slots__ = ("expositions", "followers", "is_init")
    attribute_name = "__exposition_information"
    def __init__(self):
        self.expositions = []
        self.followers = []
        self.is_init = False

    @classmethod
    def add_follower(cls, id_obj, function):
        attribute = cls._set_open_method_attribute(function)
        attribute.followers.append(id_obj)

    @classmethod
    def add_exposition(cls, id_obj, function):
        attribute = cls._set_open_method_attribute(function)
        attribute.expositions.append(id_obj)

    @classmethod
    def _set_open_method_attribute(cls, function):
        attribute = getattr(function, cls.attribute_name, None)
        if attribute is None:
            attribute = cls()
            setattr(function, cls.attribute_name, attribute)
        return attribute


def following_function(identifier, guard_condition=_dummy):
    id_obj = Identifier(identifier, guard_condition=guard_condition)
    def _(f):
        RootTransporter.regist_follower(id_obj, _weakfunction_ref(f))
        return f
    return _


def following(identifier, guard_condition=_dummy):
    id_obj = Identifier(identifier, guard_condition=guard_condition)
    def _(f):
        MethodDecorator.add_follower(id_obj, f)
        return f
    return _


def auto_twitter(identifier, entry=False, exit=False):
    id_obj = Identifier(identifier)
    def _(func):
        if entry == exit:
            entry_id = Identifier(id_obj, "entry")
            exit_id = Identifier(id_obj, "exit")
            @functools.wraps(func)
            def __(*args, **kwargs):
                RootTransporter.twitter(entry_id, args, kwargs)
                func(*args, **kwargs)
                RootTransporter.twitter(exit_id, args, kwargs)
            return __
        elif exit or id_obj.action is not None and "exit" in id_obj.action:
            id_obj.action = set(["exit"])
            @functools.wraps(func)
            def __(*args, **kwargs):
                func(*args, **kwargs)
                RootTransporter.twitter(id_obj, args, kwargs)
            return __
        elif entry or id_obj.action is not None and "entry" in id_obj.action:
            id_obj.action = set(["entry"])
            @functools.wraps(func)
            def __(*args, **kwargs):
                RootTransporter.twitter(id_obj, args, kwargs)
                func(*args, **kwargs)
            return __
        return func
    return _


def twitter_exception(identifier, reraise=False, with_traceback=True):
   id_obj = Identifier(identifier)
   def _(func):
       @functools.wraps(func)
       def __(*args, **kwargs):
           try:
               func(*args, **kwargs)
           except:
               t, v, tb = sys.exc_info()
               if with_traceback:
                   message = traceback.format_exception(t, v, tb)
               else:
                   message = traceback.format_exception_only(t, v)
               _mqas.twitter(id_obj, [message],
                             {"type":t, "value":v, "traceback":tb})
               if reraise:
                   raise
       return __
   return _


class Follower(type):
    """Metaclass for definition of follower instance methods.

    use like this:

      class Logger(metaclass=Follower):
          @following("function:call")
          def log_function_call(self, message):
              ...

          @classmethod
          @following_function("error:raised")
          def show_error(cls, message):
              ...

          @staticmethod
          @following_function("thread:created")
          def log_thread(message):
              ...
    """
    def __new__(cls, name, bases, dict):
        """Create exposition point for classmethod."""
        newtype = type.__new__(cls, name, bases, dict)
        for method in _get_attributes(newtype, types.MethodType):
            attribute = getattr(method, MethodDecorator.attribute_name, None)
            if attribute is None:
                continue
            for id_obj in attribute.followers:
                RootTransporter.regist_follower(id_obj, _weakmethod_ref(method))
            attribute.is_init = True
        return newtype

    def __call__(cls, *args):
        """Create exposition point for instancemethod."""
        instance = type.__call__(cls, *args)
        for method in _get_attributes(instance, types.MethodType):
            attribute = getattr(method, MethodDecorator.attribute_name, None)
            if attribute is None:
                continue
            if attribute.is_init:
                continue
            for id_obj in attribute.followers:
                RootTransporter.regist_follower(id_obj, _weakmethod_ref(method))
        return instance


def regist_method(identifier, method):
    id_obj = Identifier(identifier)
    RootTransporter.regist_follower(id_obj, _weakmethod_ref(method))


def regist_function(identifier, function):
    id_obj = Identifier(identifier)
    RootTransporter.regist_follower(id_obj, _weakfunction_ref(function))


class Message:
    def __init__(self, id_obj, args, kwargs, counter):
        self._id_obj = id_obj
        self._args = args
        self._kwargs = kwargs
        self._counter = counter
        self.__dict__.update(kwargs)

    @property
    def name(self):
        return self._id_obj.name

    @property
    def action(self):
        return self._id_obj.action

    @property
    def args(self):
        return self._args

    @property
    def kwargs(self):
        return self._kwargs

    @property
    def id(self):
        return self._id_obj.id_str()

    def twitter(self, id_obj, **kwargs):
        RootTransporter.twitter(id_obj, kwargs, self._counter-1)

    def apply(self, func):
        """Utility method to enease method call deligation.

        You can use like this:
          @exposition("result", "show")
          def show_result(score):
              print("my score:", score)
              # get 2 kwparams
              twitter("result", "show", username="shibu", score=score) 

          def log(username, score): # same params with exposition point.
              ...

          @following("result", "show")
          def log_score_exposition(message):
              message.apply(log) # use this!

        """
        return func(**self.kwargs)


def set_multiplicity(number):
    if number != 0:
        ThreadPool.make_thread_pool(number)
    else:
        ThreadPool.clear_thread_pool()


def show_followers():
    methods = []
    for id_obj, function_wrapper in RootTransporter.get_valid_followers():
        for action in id_obj.action:
            methods.append("%s:%s" % (id_obj.name, action))
    return sorted(set(methods))


def show_network():
    network = {}
    nodes = []
    edges = []
    classobjs = set()

    nodetype = {"class":{"shape":"box3d", "bgcolor":"#C1E4FF",
                         "pencolor":"#358ACC"},
                "joint":{"shape":"none"}}

    for name, action, method in RootTransporter.get_valid_expositions():
        key = (name, action)
        net = network.setdefault(key, {"follower":[], "exposition":[]})
        if isinstance(method, types.MethodType):
            classobjs.add(method.__self__.__class__)
            net["exposition"].append(id(method.__self__.__class__))
        else:
            net["exposition"].append(None)

    for name, action, method in RootTransporter.get_valid_followers():
        key = (name, action)
        net = network.setdefault(key, {"follower":[], "exposition":[]})
        if isinstance(method, types.MethodType):
            classobjs.add(method.__self__.__class__)
            net["follower"].append(id(method.__self__.__class__))
        else:
            net["follower"].append(None)

    for classobj in classobjs:
        nodes.append(["class", id(classobj), classobj.__name__])

    for key in network:
        followers = network[key]["follower"]
        expositions = network[key]["exposition"]
        if len(followers) == 1 and len(expositions) == 1:
            edges.append([expositions[0], followers[0],
                          {"label": "%s:%s"%key}])
        else:
            nodes.append(["joint", id(classobj), "%s:%s"%key])
            for exposition in expositions:
                edges.append([exposition, id(classobj),{"label":""}])
            for follower in followers:
                edges.append([id(classobj), follower,{"label":""}])
    result = ["digraph {"]

    for typename, objid, label in nodes:
        params = ['label="%s"' % label]
        params += ['%s="%s"' % item for item in nodetype[typename].items()]
        result.append("  %s [%s];" % (objid, ", ".join(params)))
    for start, end, params in edges:
        param_str = ['%s="%s"' % item for item in params.items()]
        result.append("  %s -> %s [%s];" % (start, end, ", ".join(param_str)))
    result.append("}")
    return "\n".join(result)


def twitter(identifier, *args, **kwargs):
    RootTransporter.twitter(Identifier(identifier), args, kwargs)


class MessageQueueReceiver:
    def __init__(self, url):
        self.servers = {}
        self.my_url = url

    def _add_connection(self, url):
        proxy = xmlrpc.client.ServerProxy(url)
        self.servers[url] = proxy
        proxy.connect(self.my_url)

    def connect(self, url):
        self.servers[url] = xmlrpc.client.ServerProxy(url)
        return True

    def quit(self):
        global _stop_server
        _stop_server = True
        return True

    def send_twitter(self, id, args, kwargs):
        print("twitter from other: %s args=%s, kwargs=%s" % (id, args, kwargs))
        RootTransporter.twitter_local(Identifier(id), args, kwargs)
        return True

    def _twitter_to_other_process(self, id_obj, args, kwargs):
        for server in self.servers.values():
            server.send_twitter(id_obj.id_str(), args, kwargs)
        return True


class _PriorityQueue(queue.Queue):
    # http://code.activestate.com/recipes/87369/
    def _init(self, maxsize):
        self.maxsize = maxsize
        self.queue = []

    def _put(self, item):
        return heapq.heappush(self.queue, item)

    def _get(self):
        return heapq.heappop(self.queue)

    def _receive_message(self, message, priority=10):
        self.put((priority, message))


Empty = queue.Empty


class QueueManager(object):
    _queues = {}
    @classmethod
    def regist_queue(cls, filter_id, shared):
        queues = cls._queues.setdefault(filter_id, {})
        id_obj = Identifier(filter_id)
        if len(queues) == 0:
            queues["shared"] = None
            queues["standalones"] = []
        if shared:
            if queues["shared"] is None:
                queue_obj = _PriorityQueue(0)
                queues["shared"] = queue_obj
                RootTransporter.regist_follower(id_obj, 
                    _weakmethod_ref(queue_obj._receive_message))
            return queues["shared"]
        else:
            queue_obj = _PriorityQueue(0)
            queues["standalones"].append(queue_obj)
            RootTransporter.regist_follower(id_obj, 
                _weakmethod_ref(queue_obj._receive_message))
            return queue_obj


class Queue(object):
    def __init__(self, filter_id, shared=False):
        if filter_id is not None:
            self._backend_queue = QueueManager.regist_queue(filter_id, shared)
        else:
            self._backend_queue = None

    def empty(self):
        return self._backend_queue.empty()

    def full(self):
        return self._backend_queue.full()

    def qsize(self):
        return self._backend_queue.qsize()

    def get(self, block=True, timeout=None):
        value = self._backend_queue.get(block, timeout)
        if value is None:
            return value
        return value[1]

    def get_nowait(self):
        return self.get(block=False)



# -*- coding: utf-8 -*-

import unittest

berrymq = None
import time

class ThreadPoolTest(unittest.TestCase):
    def test_create1(self):
        self.assertRaises(ValueError, 
            berrymq.ThreadPool.make_thread_pool, -1)

    def test_create2(self):
        berrymq.ThreadPool.make_thread_pool(5)
        self.assertEqual(5, len(berrymq.ThreadPool._pool))
        self.assert_(not berrymq.ThreadPool.empty())
        berrymq.ThreadPool.stop_thread_pool()
        self.assertEqual(0, len(berrymq.ThreadPool._pool))
        self.assert_(berrymq.ThreadPool.empty())

    def test_recreate1(self):
        berrymq.ThreadPool.make_thread_pool(3)
        self.assertEqual(3, len(berrymq.ThreadPool._pool))
        berrymq.ThreadPool.make_thread_pool(5)
        self.assertEqual(5, len(berrymq.ThreadPool._pool))
        berrymq.ThreadPool.stop_thread_pool()
        self.assertEqual(0, len(berrymq.ThreadPool._pool))

    def test_recreate2(self):
        berrymq.ThreadPool.make_thread_pool(5)
        self.assertEqual(5, len(berrymq.ThreadPool._pool))
        berrymq.ThreadPool.make_thread_pool(5)
        self.assertEqual(5, len(berrymq.ThreadPool._pool))
        berrymq.ThreadPool.stop_thread_pool()
        self.assertEqual(0, len(berrymq.ThreadPool._pool))

    def test_recreate3(self):
        berrymq.ThreadPool.make_thread_pool(5)
        self.assertEqual(5, len(berrymq.ThreadPool._pool))
        berrymq.ThreadPool.make_thread_pool(3)
        time.sleep(1)
        berrymq.ThreadPool.clear_thread_pool()
        self.assertEqual(3, len(berrymq.ThreadPool._pool))
        berrymq.ThreadPool.stop_thread_pool()
        self.assertEqual(0, len(berrymq.ThreadPool._pool))


class PrallelWorkTest(unittest.TestCase):
    def test_parallel_working(self):
        """parallel test(not parallel version)
        """
        call_history = []
        @berrymq.auto_twitter("sample1:entry")
        def exp():
            call_history.append("exp_1")
            time.sleep(3)
            call_history.append("exp_2")

        @berrymq.following_function("sample1:entry")
        def com1(message):
            time.sleep(1)
            call_history.append("com1_1")
            time.sleep(3)
            call_history.append("com1_2")

        @berrymq.following_function("sample1:entry")
        def com2(message):
            time.sleep(2)
            call_history.append("com2_1")
            time.sleep(3)
            call_history.append("com2_2")
        exp()
        self.assertEqual(["com1_1", "com1_2", "com2_1",
                          "com2_2", "exp_1", "exp_2"],
                          call_history)

    def test_parallel_working2(self):
        """parallel test
        
              0. 1. 2. 3. 4. 5.
        exp : *        *
        com1:    *        *
        com2:       *        *
        """
        berrymq.set_multiplicity(3)
        call_history = []
        @berrymq.auto_twitter("sample2:entry")
        def exp():
            call_history.append("exp_1")
            time.sleep(3)
            call_history.append("exp_2")

        @berrymq.following_function("sample2:entry")
        def com1(message):
            time.sleep(1)
            call_history.append("com1_1")
            time.sleep(3)
            call_history.append("com1_2")
            
        @berrymq.following_function("sample2:entry")
        def com2(message):
            time.sleep(2)
            call_history.append("com2_1")
            time.sleep(3)
            call_history.append("com2_2")

        exp()
        time.sleep(4)
        berrymq.set_multiplicity(0)
        self.assertEqual(["exp_1", "com1_1", "com2_1",
                          "exp_2", "com1_2", "com2_2"],
                         call_history)


def test_setup(berrymq_module):
    global berrymq
    berrymq = berrymq_module
    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.makeSuite(ThreadPoolTest))
    test_suite.addTest(unittest.makeSuite(PrallelWorkTest))
    return test_suite



# -*- coding: utf-8 -*-

from .berrymq import (following,
                      following_function,
                      auto_twitter,
                      twitter_exception,
                      cond,
                      Follower,
                      set_multiplicity,
                      twitter,
                      regist_method,
                      regist_function,)

from .connect import (init_connection,
                      interconnect,
                      connect_oneway,
                      connect_via_queue,
                      close_connection,
                      send_message,
                      get, get_nowait)



from .. import berrymq
import os
import glob
import time
import threading

class FileObserver(object):
    def __init__(self, target_dir, id_name, interval=5):
        self.id_name = id_name
        self.target_dir = target_dir
        self.interval = interval
        self.fileinfo = self._get_fileinfo()
        self.thread = threading.Thread(target=self._checkdir)
        self.thread.setDaemon(True)
        self.running = True
        self.thread.start()
        
    def stop(self):
        self.running = False

    def _checkdir(self):
        while self.running:
            time.sleep(self.interval)
            new_info = self._get_fileinfo()
            old_info = self.fileinfo
            newfiles = set(new_info.keys())
            oldfiles = set(old_info.keys())
            for created_file in (newfiles - oldfiles):
                berrymq.twitter("%s:created" % self.id_name, created_file)
            for remove_file in (oldfiles - newfiles):
                berrymq.twitter("%s:removed" % self.id_name, remove_file)
            for remain_file in (oldfiles & newfiles):
                if new_info[remain_file] != old_info[remain_file]:
                    berrymq.twitter("%s:modified" % self.id_name, remain_file)
            self.fileinfo = new_info

    def _get_fileinfo(self):
        result = {}
        for filename in glob.glob(self.target_dir):
            result[filename] = os.path.getmtime(filename)
        return result

        



# -*- coding: utf-8 -*-

from . import netgrowl
import berrymq
from socket import AF_INET, SOCK_DGRAM, socket

class GrowlAdapter(object):
    def __init__(self, id_filter, application_name="berryMQ"):
        berrymq.regist_method(id_filter, self.listener)
        self.addr = ("localhost", netgrowl.GROWL_UDP_PORT)
        self.socket = socket(AF_INET,SOCK_DGRAM)
        self.application_name = application_name
        packet = netgrowl.GrowlRegistrationPacket(application_name)
        packet.addNotification()
        self.socket.sendto(packet.payload(), self.addr)

    def format(self, message):
        argstr = ", ".join([str(arg) for arg in message.args])
        kwargstr = ", ".join(["%s:%s" % (str(key), str(value))
                             for key, value in sorted(message.kwargs.items())])
        return "[%s], {%s}" % (argstr, kwargstr)
        
    def listener(self, message):
        desc = self.format(message)
        packet = netgrowl.GrowlNotificationPacket(
            self.application_name, title=message.id, description=desc)
        self.socket.sendto(packet.payload(), self.addr)



from .. import berrymq
import os
import time
import threading

class IntervalTimer(object):
    def __init__(self, id_name, interval):
        self.id_name = id_name
        self.interval = interval
        self.thread = threading.Thread(target=self._notify)
        self.thread.setDaemon(True)
        self.running = True
        self.thread.start()

    def _checkdir(self):
        while self.running:
            time.sleep(self.interval)
            berrymq.twitter("%s:tick" % self.id_name)

    def stop(self):
        self.running = False



# -*- coding: utf-8 -*-

from .berrymq import (following,
                      following_function,
                      auto_twitter,
                      cond,
                      Follower,
                      twitter)



import sys
from setuptools import setup, find_packages 

__version__ = open("VERSION").read().strip()

setup(name = 'berrymq',
      version = __version__,
      author = 'SHIBUKAWA Yoshiki',
      author_email = 'yoshiki at shibu.jp',
      url = 'http://berrymq.org',
      description = 'Small message queue system for building applications',
      long_description = ('Programmer friendly message queue module. '
                        'It supports push/pull API, thread pool'),
      keywords = "python ruby message queue mq",
      classifiers = [
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Programming Language :: Python"
      ],
      license = "MIT License",
      packages = find_packages(),
      install_requires = [
      ])




import time
import uuid
from . import berrymq
from .jsonrpc import client
from .jsonrpc import server


class Timeout(object):
    def __init__(self, token):
        self.token = token


class UnInitializedServer(RuntimeError): pass


class AlreadyConnected(RuntimeError): pass


class NotConnectToServer(RuntimeError): 
    """This node has never been initialized as client"""
    pass


class UnAvialbleFunction(RuntimeError):
    """get(), get_nowait() are available if style03 client."""
    pass


class Connection(object):
    """This supports type02 connection and base class of other class.
    
    This object is used only client side.
    """
    def __init__(self, ttl=None, client=False):
        self.last_access = time.time()
        self.token = None
        self.ttl = ttl
        self.proxy = None
        self.addr = None
        self.client = client
        
    def _generate_token(self):
        return str(uuid.uuid1())

    def connect(self, addr):
        """This method is called at client side."""
        self.addr = addr
        self.proxy = client.ServerProxy("http://%s:%d" % addr)
        self.token = self.proxy.connect_oneway(self.ttl)
        return self.token

    def check_ttl(self):
        if not self.ttl:
            return
        now = time.time()
        if now - self.last_access > self.ttl:
            raise Timeout(self.token)
        self.last_access = now

    def send_message(self, identifier, args, kwargs):
        if self.proxy is not None and self.client:
            self.proxy.send_message(self.token, identifier, args, kwargs)

    def forward_message(self, identifier, args, kwargs):
        pass


class Interconnection(Connection):
    """This class supports type01 connection.
    
    This object is used both client and server sides.
    """
    def __init__(self, ttl, client=False):
        super(Interconnection, self).__init__(ttl, client)

    def connect_at_client(self, addr):
        """This method is called at client side."""
        self.addr = addr
        self.token = self._generate_token()
        self.proxy = client.ServerProxy("http://%s:%d" % addr)
        my_addr = ConnectionPoint.get_addr()
        self.proxy.interconnect(my_addr, self.token, self.ttl)

    def connect_at_server(self, addr, token):
        self.addr = addr
        self.proxy = client.ServerProxy("http://%s:%d" % addr)
        self.token = token

    def forward_message(self, identifier, args, kwargs):
        if self.proxy is not None:
            self.proxy.send_message(self.token, identifier, args, kwargs)

    def send_message(self, identifier, args, kwargs):
        pass


class QueueConnection(Connection):
    """This class supports type03 server side connection"""
    def __init__(self, identifier, ttl):
        super(QueueConnection, self).__init__(ttl, client=False)
        self.queue = berrymq.Queue(identifier)
        self.token = self._generate_token()

    def receive_get_request(self, block, timeout):
        self.check_ttl()
        return self.queue.get(block, timeout)

    def receive_get_nowait(self):
        self.check_ttl()
        return self.queue.get_nowait()

    def send_message(self, identifier, args, kwargs):
        pass


class QueueClientConnection(Connection):
    """This class supports type03 client side connection"""
    def __init__(self, ttl):
        super(QueueClientConnection, self).__init__(ttl, client=True)

    def connect(self, addr, identifier):
        """This method is called at client side"""
        url = "http://%s:%d" % addr
        self.proxy = client.ServerProxy(url)
        self.token = self.proxy.connect_via_queue(identifier, self.ttl)
        self.addr = addr
        return self.token

    def send_get(self, block, timeout):
        return self.proxy.get(self.token, block, timeout)

    def send_get_nowait(self):
        return self.proxy.get_nowait(self.token)


class ExportedFunctions(object):
    """Exported JSON-RPC Server"""
    def interconnect(self, addr, his_token, ttl):
        connection = Interconnection(ttl, client=False)
        connection.connect_at_server(tuple(addr), his_token)
        return ConnectionPoint.append(connection)

    def connect_oneway(self, ttl=1000):
        return ConnectionPoint.append(Connection(ttl))

    def connect_via_queue(self, identifier, ttl):
        return ConnectionPoint.append(QueueConnection(identifier, ttl))

    def close_connection(self, token, addr=None):
        """
        :todo: implement
        """
        return "ok"

    def send_message(self, token, identifier, args, kwargs):
        return ConnectionPoint.receive_message(token, identifier, args, kwargs)

    def get(self, token, block, timeout):
        return ConnectionPoint.receive_get(token, block, timeout)

    def get_nowait(self, token):
        return ConnectionPoint.receive_get(token, False, 0)

    def __twitter_to_other_process(self, id_obj, args, kwargs):
        for server in self.servers.itervalues():
            server.send_twitter(id_obj.id_str(), args, kwargs)
        return True


class ConnectionPoint(object):
    """This class manage network connection.
    """
    _server = None
    _addr = None
    _connections = {}
    _url_to_token = {}

    @classmethod
    def init(cls, addr):
        if cls._server:
            raise RuntimeError("server is already initialized")
        cls._server = server.SimpleJSONRPCServer(addr)
        cls._server.register_instance(ExportedFunctions())
        cls._server.serve_forever(in_thread=True)
        cls.regist_exchanger()
        cls._addr = addr
        
    @classmethod
    def get_addr(cls):
        if cls._addr is None:
            raise RuntimeError("Please call init_connection() first.")
        return cls._addr

    @classmethod
    def regist_exchanger(cls):
        berrymq.RootTransporter._connections = cls

    @classmethod
    def clear_exchanger(cls):
        berrymq.RootTransporter._connections = None

    @classmethod
    def append(cls, connection):
        """Regist connection information.

        This method is work at both side of client and server.
        If you run this server at client side, turn on client parameter.

        @param connection: Connection information
        @type  connection: a kind of Connection's instance
        @param client: set True if this node is client
        @type  client: bool
        @return: token
        @rtype:  str
        """
        cls._connections[connection.token] = connection
        if connection.addr:
            cls._url_to_token[connection.addr] = connection.token
        return connection.token

    @classmethod
    def _allow_token(cls, token):
        """test method"""
        cls._connections[token] = "test"

    @classmethod
    def remove_connection(addr, url):
        token = cls._url_to_token.get(addr)
        if token:
            del cls._url_to_token[addr]
            del cls._connections[token]

    @classmethod
    def close(cls):
        if cls._server is not None:
            cls._server.shutdown()
        cls._server = None
        cls._connections = {}
        cls._url_to_token = {}
        cls.clear_exchanger()

    @classmethod
    def send_message(cls, identifier, args, kwargs):
        for connection in cls._connections.values():
            connection.send_message(identifier, args, kwargs)

    @classmethod
    def forward_message(cls, identifier, args, kwargs, except_token=None):
        """Forward message to style01 connection clients/server
        @todo: ttl check
        """
        for token, connection in cls._connections.items():
            if token == except_token:
                continue
            connection.forward_message(identifier, args, kwargs)

    @classmethod
    def send_get(cls, block, timeout):
        for connection in cls._connections.values():
            if isinstance(connection, QueueClientConnection):
                idstr, args, kwargs = connection.send_get(block, timeout)
                return berrymq.Message(berrymq.Identifier(idstr), 
                                       args, kwargs, 100)
        raise NotConnectToServer()

    @classmethod
    def receive_get(cls, token, block, timeout):
        connection = cls._connections.get(token)
        if not isinstance(connection, QueueConnection):
            return "invalid token"
        try:
            connection.check_ttl()
        except Timeout as e:
            del cls._connections[e.token]
            return "timeout"
        message = connection.queue.get(block, timeout)
        return [message.id, message.args, message.kwargs]

    @classmethod
    def receive_message(cls, token, identifier, args, kwargs):
        connection = cls._connections.get(token)
        if connection is None:
            return "invalid token"
        elif isinstance(connection, Connection):
            try:
                connection.check_ttl()
            except Timeout as e:
                del self.connections[e.token]
                return "timeout"
        id_obj = berrymq.Identifier(identifier)
        berrymq.RootTransporter.twitter_local(id_obj, args, kwargs)
        cls.forward_message(identifier, args, kwargs, token)
        return "ok"


# berryMQ API


def init_connection(addr=("localhost", 0)):
    ConnectionPoint.init(addr)


def interconnect(addr, ttl=1000):
    connection = Interconnection(ttl, client=True)
    connection.connect_at_client(addr)
    ConnectionPoint.append(connection)


def connect_oneway(addr, ttl=1000):
    connection = Connection(ttl, client=True)
    connection.connect(addr)
    ConnectionPoint.append(connection)


def connect_via_queue(addr, identifier, ttl=1000):
    connection = QueueClientConnection(ttl)
    token = connection.connect(addr, identifier)
    ConnectionPoint.append(connection)


def send_message(identifier, *args, **kwargs):
    ConnectionPoint.send_message(identifier, args, kwargs)


def get(block=True, timeout=1000):
    return ConnectionPoint.send_get(block, timeout)


def get_nowait():
    return ConnectionPoint.send_get(block=False, timeout=0)


def close_connection(url=None):
    if url:
        ConnectionPoint.remove_connection(url)
    else:
        ConnectionPoint.close()
